//
//  ViewController.swift
//  acapelacloud-ios-mac-demo
//
//  Created by sebastienboero on 09/10/2019.
//  Copyright © 2019 acapela. All rights reserved.
//

import UIKit
import AVFoundation
import AVKit


class ViewController: UIViewController {
    
    @IBOutlet weak var voice_button: UIButton!
    @IBOutlet weak var text_view: UITextView!
    @IBOutlet weak var account_view: UITextView!

    var voices: [String] = [];
    var voice_index = 0
    
    // Fill with your credentials
    var email = ""
    var password = ""
    
    var token = ""
    
 
    var player: AVPlayer!
    var playerItem: AVPlayerItem!
    var timeObserverToken: Any?
    var startDate: Date?
    
    
    func generate_audio_file(completion: @escaping (String)->Void) {
          
        var urlComponents = URLComponents()
        urlComponents.scheme = "https"
        urlComponents.host = "www.acapela-cloud.com"
        urlComponents.path = "/api/command"
        urlComponents.queryItems = [
           URLQueryItem(name: "output", value: "storage"),
           URLQueryItem(name: "voice", value: voices[voice_index]),
           URLQueryItem(name: "text", value:  text_view.text),
           URLQueryItem(name: "type", value: "mp3"),
           URLQueryItem(name: "token", value:   self.token),
        ]

        let url = urlComponents.url!

        var request = URLRequest(url: url)
        request.httpMethod = "GET"
        request.setValue("application/json; charset=utf-8", forHTTPHeaderField: "Content-Type")
        request.setValue("application/json; charset=utf-8", forHTTPHeaderField: "Accept")

        let task = URLSession.shared.dataTask(with: request) { data, response, error in
         
            guard let data = data, let response = response as? HTTPURLResponse, (200 ..< 300) ~= response.statusCode,
                error == nil else {
                    return
            }
            
            let paths = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask)
            let audiofilepath = paths[0].appendingPathComponent("audio.mp3")
            print(audiofilepath)

            do {
                try data.write(to: audiofilepath)
            } catch {
                print("error")

            }
            
            print(audiofilepath)
         
        }

        task.resume()
      
      }
    
    

    func play() {
        
        DispatchQueue.main.async {
            self.generate_audio_file() {response in
            }
        }
        
        stop()
        
        startDate = Date()

        var urlComponents = URLComponents()
        urlComponents.scheme = "https"
        urlComponents.host = "www.acapela-cloud.com"
        urlComponents.path = "/api/command"
        urlComponents.queryItems = [
            URLQueryItem(name: "output", value: "stream"),
            URLQueryItem(name: "voice", value: voices[voice_index]),
            URLQueryItem(name: "text", value:  text_view.text),
            URLQueryItem(name: "type", value: "mp3"),
            URLQueryItem(name: "token", value:   self.token),
        ]

        let url = urlComponents.url!.absoluteString

        playerItem = AVPlayerItem(url: URL(string: url)!)
        player = AVPlayer(playerItem: playerItem)

        self.addPeriodicTimeObserver();
        let playerLayer = AVPlayerLayer(player: player)
        playerLayer.frame = self.view.bounds
        self.view.layer.addSublayer(playerLayer)
        player.play()

    }
    
    
    func addPeriodicTimeObserver() {

        let timeScale = CMTimeScale(NSEC_PER_SEC)
        let time = CMTime(seconds: 0.5, preferredTimescale: timeScale)
        timeObserverToken = player.addPeriodicTimeObserver(forInterval: time, queue: .main) {
            [weak self] time in
            print("audio started")
            print(Date().timeIntervalSince((self!.startDate)!) * 1000)
            self!.removePeriodicTimeObserver()
        }
    }
    

    func removePeriodicTimeObserver() {
        if let timeObserverToken = timeObserverToken {
            player.removeTimeObserver(timeObserverToken)
            self.timeObserverToken = nil
        }
    }
    
    
    func stop() {
        if (player != nil) {
            player.pause()
            player = nil
        }
    }
    
    func pause() {
        if (player != nil) {
            player.pause()
        }
    }
    
    func resume() {
        if (player != nil) {
            player.play()
        }
    }
    
    func login(completion: @escaping (String)->Void) {
        
        let url = URL(string: "https://www.acapela-cloud.com/api/login/")!
        var request = URLRequest(url: url)
        request.httpMethod = "POST"
        request.setValue("application/json; charset=utf-8", forHTTPHeaderField: "Content-Type")
        request.setValue("application/json; charset=utf-8", forHTTPHeaderField: "Accept")
        
        let json: [String: Any] = ["email": email, "password": password]
        let jsonData = try? JSONSerialization.data(withJSONObject: json)
        request.httpBody = jsonData
        
        let task = URLSession.shared.dataTask(with: request) { data, response, error in
            guard let data = data, error == nil else {
                print(error?.localizedDescription ?? "No data")
                return
            }
            let responseJSON = try? JSONSerialization.jsonObject(with: data, options: [])
            
            if let responseJSON = responseJSON as? [String: Any] {
                
                for (key, value) in responseJSON {
                    print(key, value)
                    if (key == "token") {
                        self.token = value as! String
                        completion(self.token)
                    }
                }
            }
        }
        
        task.resume()
    
    }
    
    
    func account_info(token : String, completion: @escaping (String)->Void) {
        
        let url = URL(string: "https://www.acapela-cloud.com/api/account/")!
        var request = URLRequest(url: url)
        request.httpMethod = "GET"
        request.setValue("application/json; charset=utf-8", forHTTPHeaderField: "Content-Type")
        request.setValue("application/json; charset=utf-8", forHTTPHeaderField: "Accept")
        request.setValue("Token " + token, forHTTPHeaderField: "Authorization")
        
        
        let task = URLSession.shared.dataTask(with: request) { data, response, error in
            guard let data = data, error == nil else {
                print(error?.localizedDescription ?? "No data")
                return
            }
            let responseJSON = try? JSONSerialization.jsonObject(with: data, options: [])
            
            if let responseJSON = responseJSON as? [String: Any] {
                
               print(responseJSON)
                
                for (key, value) in responseJSON {
                    if (key == "voices") {
                        print(key, value)
                        let voices = value as! [Dictionary<String, AnyObject>]
                        
                        // Get voice names
                        for items in voices {
                            //print(items["name"] as! String)
                            self.voices.append(items["name"] as! String)
                        }
                    } else {
                        DispatchQueue.main.async() {
                            self.account_view.text += "\(key) - \(value) \n"
                        }
                    }
                }
                completion("")
            }
        }
        
        task.resume()
        
    }
   
    @IBAction func select_voice(_ sender: Any) {
        
        if (voices.count != 0) {
            voice_index += 1
            
            if voice_index >= voices.count {
                voice_index = 0
            }
            
            voice_button.setTitle(voices[voice_index], for: .normal)
        }
    }
    
    @IBAction func speak(_ sender: Any) {
        play()
    }
  
    @IBAction func pause(_ sender: Any) {
        pause()
    }
    
    @IBAction func resume(_ sender: Any) {
        resume()
    }
    
    @IBAction func stop(_ sender: Any) {
        stop()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let audioSession = AVAudioSession.sharedInstance()
        do {
            try audioSession.setCategory(AVAudioSession.Category.playback)
        }
        catch {
            print("Setting category to AVAudioSessionCategoryPlayback failed.")
        }
        
        login() {response in
            self.token = response
            self.account_info(token:self.token) {response in
                DispatchQueue.main.async {
                    self.voice_button.setTitle(self.voices[0] , for: .normal)
                    
                }
                
            }
        }
    }
}

